import React, { memo, Fragment } from 'react';
import MultiValue from './MultiValue';
import styled from 'styled-components';
import { isArrayWithLength } from '../utils';
import { SINGLE_VALUE_BASE_STYLE } from '../constants/styled';
const SingleValue = styled.div `
  ${SINGLE_VALUE_BASE_STYLE}
  max-width: calc(100% - 0.5rem);
`;
const Placeholder = styled.div `
  ${SINGLE_VALUE_BASE_STYLE}
  color: ${({ theme }) => theme.color.placeholder};
`;
const Value = memo(({ isMulti, inputValue, placeholder, selectedOption, focusedMultiValue, renderOptionLabel, renderMultiOptions, removeSelectedOption }) => {
    if (inputValue &&
        (!isMulti || (isMulti && (!isArrayWithLength(selectedOption) || renderMultiOptions)))) {
        return null;
    }
    if (!isArrayWithLength(selectedOption)) {
        return React.createElement(Placeholder, null, placeholder);
    }
    if (!isMulti) {
        return React.createElement(SingleValue, null, renderOptionLabel(selectedOption[0].data));
    }
    return (React.createElement(Fragment, null, renderMultiOptions
        ? renderMultiOptions({ selected: selectedOption, renderOptionLabel })
        : selectedOption.map(({ data, value }) => (React.createElement(MultiValue, { key: value, data: data, value: value, renderOptionLabel: renderOptionLabel, isFocused: value === focusedMultiValue, removeSelectedOption: removeSelectedOption })))));
});
Value.displayName = 'Value';
export default Value;
